pro prepare_proj_fine, file_scalars=file_scalars, xdim=xdim, ydim=ydim, $
                       scan_next=scan_next, subscan_next=subscan_next, $
                       ind_scans=ind_scans, ind_subscans=ind_subscans, $
                       nzadata=nzadata, cut_pcal=cut_pcal


;;; projection vectors to speed up mapping of time and bolometer indices onto fine pixels

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; copyright 2007-2011 H\'el\`ene Roussel
;;; roussel@iap.fr
;;; Institut d'Astrophysique de Paris - Universit\'e Pierre et Marie Curie, France

;;; download and documentation: http://www2.iap.fr/users/roussel/herschel/

;;; Scanamorphos is distributed under the terms of the GNU General Public License
;;; (http://www.gnu.org/licenses/gpl.html).

;;; July 2010: public release version 1

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



restore, file_scalars

if (not keyword_set(xdim)) then xdim = dimx
if (not keyword_set(ydim)) then ydim = dimy


restore, file_pixcoords

indxy_beam = floor(x_bolos) + xdim * floor(y_bolos)



;;; suppress invalid samples:

restore, file_signal

wnan = where(finite(signal) lt 1 or x_bolos lt 0. or x_bolos ge xdim $
   or y_bolos lt 0. or y_bolos ge ydim, cw_nan)
signal = 0
x_bolos = 0
y_bolos = 0
if cw_nan ge 1 then indxy_beam(wnan) = -9L
wnan = 0



;;; suppress scan initial slew (PACS) and optionally non-zero acceleration data
;;; and scan start after PCAL flashes:

restore, file_times
time_s = 0
if inst eq 'pacs' then for scan = 0, nscans - 1 do begin
   w = where(flag_nza(ind_scans(scan):ind_scans(scan+1)-1) eq 1, cw)
   if w(0) gt 0 then indxy_beam(ind_scans(scan):ind_scans(scan)+w(0)-1, *) = -9L
   w = 0
endfor

if keyword_set(nzadata) then if nzadata eq 'no' then begin
   wnza = where(flag_nza eq 0, cw_nza)
   if cw_nza ge 1 then indxy_beam(wnza, *) = -9L
   wnza = 0
endif
flag_nza = 0

if keyword_set(cut_pcal) then begin
   wcut = where(cut_pcal gt 0, cw_cut)
   for c = 0, cw_cut - 1 do begin
      subs = where(ind_subscans ge ind_scans(wcut(c)))
      indxy_beam(ind_subscans(subs(0)):ind_subscans(subs(0)+1)-1, *) = -9L
   endfor
endif



;;; optionally retain a single scan or a single subscan:

if keyword_set(scan_next) then for scan = 0, nscans - 1 do if scan ne scan_next - 1 then $
   indxy_beam(ind_scans(scan):ind_scans(scan+1)-1, *) = -9L

if keyword_set(subscan_next) then for subs = 0, nsubscans - 1 do if subs ne subscan_next - 1 then $
   indxy_beam(ind_subscans(subs):ind_subscans(subs+1)-1, *) = -9L



histo_indxy = histogram(indxy_beam, reverse_indices=revind_fine, min=0., max=xdim*ydim-1)
indxy_beam = 0
wproj_fine = where(histo_indxy ge 1L, nproj_fine)
histo_indxy = 0


;;; 1D index array of imprint lower-left corner (repetitions eliminated by wproj_fine):
vectpos_startimprint = floor((wproj_fine mod xdim) - xcmin_imprint) $
      + xdim * floor(wproj_fine / xdim - xcmin_imprint)

;;; 1D index array of entire imprint, relative to the lower-left corner:
tab_ind = lindgen(dim_imprint)
sq_dim_imprint = dim_imprint^2
vectpos_imprint = reform(rebin(tab_ind, dim_imprint, dim_imprint), sq_dim_imprint) $
   + xdim * rebin(tab_ind, sq_dim_imprint, /sample)



save, file=file_projfine, wproj_fine, nproj_fine, revind_fine, $
   vectpos_startimprint, vectpos_imprint, /xdr

end
